@@ -1,7 +1,7 @@
 package Data::CSel;
 
-our $DATE = '2016-05-31'; # DATE
-our $VERSION = '0.08'; # VERSION
+our $DATE = '2016-09-01'; # DATE
+our $VERSION = '0.09'; # VERSION
 
 use 5.020000;
 use strict;
@@ -114,12 +114,12 @@ our $RE =
               (?<FILTER>
                   (?{ [$^R, {}] })
                   (
-                      (?&ATTR_SELECTOR) # [[$^R, {}], [$attr, $op, $val]]
+                      (?&ATTR_SELECTOR) # [[$^R0, {}], [$attr, $op, $val]]
                       (?{
                           $^R->[0][1]{type}  = 'attr_selector';
                           $^R->[0][1]{attr}  = $^R->[1][0];
-                          $^R->[0][1]{op}    = $^R->[1][1] if @{$^R->[1]} > 1;
-                          $^R->[0][1]{value} = $^R->[1][2] if @{$^R->[1]} > 2;
+                          $^R->[0][1]{op}    = $^R->[1][1] if defined $^R->[1][1];
+                          $^R->[0][1]{value} = $^R->[1][2] if @{ $^R->[1] } > 2;
                           $^R->[0];
                       })
                   |
@@ -148,45 +148,103 @@ our $RE =
               )
 
               (?<ATTR_SELECTOR>
-                  \[\s*((?&ATTR_NAME))\s*\]
-                  (?{ [$^R, [$^N]] })
-              |
                   \[\s*
-                  ((?&ATTR_NAME))
-                  (?{ [$^R, [$^N]] })
-
-                  (
-                      \s*(?:=~|!~)\s* |
-                      \s*(?:!=|<>|>=?|<=?|==?)\s* |
-                      \s+(?:eq|ne|lt|gt|le|ge)\s+ |
-                      \s+(?:isnt|is)\s+
-                  )
+                  (?{ [$^R, []] }) # [$^R, [$subjects, $op, $literal]]
+                  (?&ATTR_SUBJECTS) # [[$^R, [{name=>$name, args=>$args}, ...]]
                   (?{
-                      my $op = $^N;
-                      $op =~ s/^\s+//; $op =~ s/\s+$//;
-                      push @{$^R->[1]}, $op;
-                      $^R;
+                      push @{ $^R->[0][1] }, $^R->[1];
+                      $^R->[0];
                   })
 
                   (?:
-                      ((?&LITERAL)) # [[$^R, [$attr, $op]], $literal]
+                      (
+                          \s*(?:=~|!~)\s* |
+                          \s*(?:!=|<>|>=?|<=?|==?)\s* |
+                          \s+(?:eq|ne|lt|gt|le|ge)\s+ |
+                          \s+(?:isnt|is)\s+
+                      )
                       (?{
-                          push @{ $^R->[0][1] }, $^R->[1];
-                          $^R->[0];
-                      })
-                  |
-                      (\w[^\s\]]*) # allow unquoted string
-                      (?{
-                          push @{ $^R->[1] }, $^N;
+                          my $op = $^N;
+                          $op =~ s/^\s+//; $op =~ s/\s+$//;
+                          $^R->[1][1] = $op;
                           $^R;
                       })
-                  )
+
+                      (?:
+                          (?&LITERAL) # [[$^R0, [$attr, $op]], $literal]
+                          (?{
+                              push @{ $^R->[0][1] }, $^R->[1];
+                              $^R->[0];
+                          })
+                      |
+                          (\w[^\s\]]*) # allow unquoted string
+                          (?{
+                              $^R->[1][2] = $^N;
+                              $^R;
+                          })
+                      )
+                  )?
                   \s*\]
               )
 
               (?<ATTR_NAME>
                   [A-Za-z_][A-Za-z0-9_]*
-                  (?:\.[A-Za-z_][A-Za-z0-9_]*)*
+              )
+
+              (?<ATTR_SUBJECT>
+                  (?{ [$^R, []] }) # [$^R, [name, \@args]]
+                  ((?&ATTR_NAME))
+                  (?{
+                      push @{ $^R->[1] }, $^N;
+                      $^R;
+                  })
+                  (?:
+                      # attribute arguments
+                      \s*\(\s*
+                      (?{
+                          $^R->[1][1] = [];
+                          $^R;
+                      })
+                      (?:
+                          (?&LITERAL)
+                          (?{
+                              push @{ $^R->[0][1][1] }, $^R->[1];
+                              $^R->[0];
+                          })
+                          (?:
+                              \s*,\s*
+                              (?&LITERAL)
+                              (?{
+                                  push @{ $^R->[0][1][1] }, $^R->[1];
+                                  $^R->[0];
+                              })
+                          )*
+                      )?
+                      \s*\)\s*
+                  )?
+              )
+
+              (?<ATTR_SUBJECTS>
+                  (?{ [$^R, []] })
+                  (?&ATTR_SUBJECT) # [[$^R, [$name, \@args]]
+                  (?{
+                      push @{ $^R->[0][1] }, {
+                          name => $^R->[1][0],
+                          (args => $^R->[1][1]) x !!defined($^R->[1][1]),
+                      };
+                      $^R->[0];
+                  })
+                  (?:
+                      \s*\.\s*
+                      (?&ATTR_SUBJECT) # [[$^R, $name, \@args]]
+                      (?{
+                          push @{ $^R->[0][1] }, {
+                              name => $^R->[1][0],
+                              (args => $^R->[1][1]) x !!defined($^R->[1][1]),
+                          };
+                          $^R->[0];
+                      })
+                  )*
               )
 
               (?<LITERAL>
@@ -378,7 +436,7 @@ sub _simpsel {
 
         if ($type eq 'attr_selector') {
 
-            my @attrs = split /\./, $f->{attr};
+            my @attrs = @{ $f->{attr} };
             my $op    = $f->{op};
             my $opv   = $f->{value};
 
@@ -386,9 +444,20 @@ sub _simpsel {
           ITEM:
             for my $o0 (@res) {
                 my $o = $o0;
-                for my $attr (@attrs) {
-                    next ITEM unless blessed($o) && $o->can($attr);
-                    $o = $o->$attr;
+                for my $i (0..$#attrs) {
+                    my $attr_name = $attrs[$i]{name};
+                    my $attr_args = $attrs[$i]{args};
+                    next ITEM unless blessed($o) && $o->can($attr_name);
+                    if ($attr_args && $i == $#attrs) {
+                        # T[meth()] means T[meth() is true]
+                        unless ($op) {
+                            $op = 'is';
+                            $opv = 1;
+                        }
+                        $o = $o->$attr_name(@$attr_args);
+                    } else {
+                        $o = $o->$attr_name;
+                    }
                 }
                 goto PASS unless $op;
                 my $val = $o;
@@ -615,7 +684,7 @@ Data::CSel - Select tree node objects using CSS Selector-like syntax
 
 =head1 VERSION
 
-This document describes version 0.08 of Data::CSel (from Perl distribution Data-CSel), released on 2016-05-31.
+This document describes version 0.09 of Data::CSel (from Perl distribution Data-CSel), released on 2016-09-01.
 
 =head1 SYNOPSIS
 
@@ -703,7 +772,31 @@ C<:not> pseudo-class (see L</"Pseudo-class">), for example:
 
 C<[ATTR OP LITERAL]> means to only select objects that have an attribute named
 C<ATTR> that has value that matches the expression specified by operator C<OP>
-and operand C<LITERAL>.
+and operand C<LITERAL>. For example:
+
+ [length > 12]
+ [is_done is true]
+ [name =~ /foo/]
+
+B<Calling methods> C<ATTR> can also be replaced by C<METH()> or C<METH(LITERAL,
+...)> to allow passing arguments to methods. Note that this specific syntax:
+
+ [METH()]
+
+does not simply mean to select objects that respond to C<METH>, but actually:
+
+ [METH() is true]
+
+For example:
+
+ # select objects that have non-zero length
+ [length()]
+
+ # while this means to select objects that have 'length' attribute
+ [length]
+
+ # select objects for which the method call returns true
+ [has_key('foo')]
 
 B<Experimental:> a chain of attributes is allowed for the attribute, for
 example:
@@ -1207,6 +1300,13 @@ convenience). See the selection object's documentation for more details.
 Parse an expression. On success, will return a hash containing parsed
 information. On failure, will return undef.
 
+=head1 FAQ
+
+=head2 Can I use csel() against a regular data structure (instead of a tree of objects)?
+
+Use L<Data::CSel::WrapStruct> to create a tree of object from the data
+structure, then perform C<csel()> on the resulting tree.
+
 =head1 HOMEPAGE
 
 Please visit the project's homepage at L<https://metacpan.org/release/Data-CSel>.
@@ -1242,6 +1342,10 @@ an HTML document: L<Mojo::DOM> (or L<DOM::Tiny>), L<jQuery>, L<pQuery>,
 L<HTML::Selector::XPath> (or via L<Web::Query>). The last two modules can also
 handle XPath expression.
 
+L<Data::CSel::WrapStruct>
+
+L<CSel::Examples>
+
 =head1 AUTHOR
 
 perlancar <perlancar@cpan.org>
